﻿using MotorPlantContracts.BindingModels;
using MotorPlantContracts.BusinessLogicsContracts;
using MotorPlantContracts.SearchModels;
using MotorPlantContracts.StoragesContracts;
using MotorPlantContracts.ViewModels;
using Microsoft.Extensions.Logging;
using MotorPlantDataModels.Enums;
using MotorPlantDataModels.Models;

namespace MotorPlantBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
		private readonly ILogger _logger;
		private readonly IOrderStorage _orderStorage;
		private readonly IShopStorage _shopStorage;
		private readonly IShopLogic _shopLogic;
		private readonly IEngineStorage _engineStorage;
		public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, IShopStorage shopStorage, IShopLogic shopLogic, IEngineStorage engineStorage)
		{
			_logger = logger;
			_orderStorage = orderStorage;
			_shopStorage = shopStorage;
			_shopLogic = shopLogic;
			_engineStorage = engineStorage;
		}

		public List<OrderViewModel>? ReadList(OrderSearchModel? model)
		{
			_logger.LogInformation("ReadList. OrderId:{Id}", model?.Id);
			var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count:{Count}", list.Count);
			return list;
		}

		private void CheckModel(OrderBindingModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (model.Count <= 0)
			{
				throw new ArgumentNullException("Количество заказов должно быть больше нуля");
			}
			if (model.Sum <= 0)
			{
				throw new ArgumentNullException("Цена заказа должна быть больше нуля");
			}
			_logger.LogInformation("Order. OrderId:{Id}. Sum:{Sum}", model.Id, model.Sum);
		}

		public bool CreateOrder(OrderBindingModel model)
		{
			CheckModel(model);
			if (model.Status != OrderStatus.Неизвестен)
			{
				return false;
			}
			model.Status = OrderStatus.Принят;
			if (_orderStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;

		}

		private bool ChangeStatus(OrderBindingModel model, OrderStatus status)
		{
			var element = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
			if (element == null)
			{
				_logger.LogWarning("Find order failed");
				return false;
			}
			if (element.Status != status - 1)
			{
				_logger.LogWarning("Status change failed");
				throw new InvalidOperationException("Невозможно перевести состояние заказа");
			}
			if (status == OrderStatus.Готов)
			{
				var engine = _engineStorage.GetElement(new EngineSearchModel() { Id = model.EngineId });
				if (engine == null)
				{
					_logger.LogWarning("Status change error. Engine not found");
					return false;
				}
				if (!CheckSupply(engine, model.Count))
				{
					_logger.LogWarning("Status change error. Shop doesnt have engines");
					return false;
				}
			}
			model.Status = status;
			if (model.Status == OrderStatus.Выдан) model.DateImplement = DateTime.Now;
			_orderStorage.Update(model);
			return true;
		}

		public bool TakeOrderInWork(OrderBindingModel model)
		{
			return ChangeStatus(model, OrderStatus.Выполняется);
		}

		public bool FinishOrder(OrderBindingModel model)
		{
			return ChangeStatus(model, OrderStatus.Готов);
		}

		public bool DeliveryOrder(OrderBindingModel model)
		{
			return ChangeStatus(model, OrderStatus.Выдан);
		}

		public bool CheckSupply(IEngineModel engine, int count)
		{
			if (count <= 0)
			{
				_logger.LogWarning("Check supply operation error. Engine count < 0");
				return false;
			}
			int sumCapacity = _shopStorage.GetFullList().Select(x => x.MaxCount).Sum();
			int sumCount = _shopStorage.GetFullList().Select(x => x.ShopEngines.Select(y => y.Value.Item2).Sum()).Sum();
			int free = sumCapacity - sumCount;

			if (free < count)
			{
				_logger.LogWarning("Check supply error. No place for new Engines");
				return false;
			}
			foreach (var shop in _shopStorage.GetFullList())
			{
				free = shop.MaxCount;
				foreach (var doc in shop.ShopEngines)
				{
					free -= doc.Value.Item2;
				}
				if (free == 0)
				{
					continue;
				}
				if (free >= count)
				{
					if (_shopLogic.MakeSupply(new()
					{
						Id = shop.Id
					}, engine, count))
					{
						count = 0;
					}
					else
					{
						_logger.LogWarning("Supply error");
						return false;
					}
				}
				else
				{
					if (_shopLogic.MakeSupply(new() { Id = shop.Id }, engine, free))
						count -= free;
					else
					{
						_logger.LogWarning("Supply error");
						return false;
					}
				}
				if (count <= 0)
				{
					return true;
				}
			}
			return false;
		}
	}
}
